@@ -1,3 +1,5 @@
+#include <tommath.h>
+#ifdef BN_MP_INIT_COPY_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
  * LibTomMath is a library that provides multiple-precision
@@ -10,13 +12,11 @@
  * The library is free for all purposes without any express
  * guarantee it works.
  *
- * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
+ * Tom St Denis, tomstdenis@gmail.com, http://math.libtomcrypt.com
  */
-#include <tommath.h>
 
 /* creates "a" then copies b into it */
-int
-mp_init_copy (mp_int * a, mp_int * b)
+int mp_init_copy (mp_int * a, mp_int * b)
 {
   int     res;
 
@@ -25,3 +25,8 @@ mp_init_copy (mp_int * a, mp_int * b)
   }
   return mp_copy (b, a);
 }
+#endif
+
+/* $Source: /cvs/libtom/libtommath/bn_mp_init_copy.c,v $ */
+/* $Revision: 1.3 $ */
+/* $Date: 2006/03/31 14:18:44 $ */
